@@ -9,28 +9,35 @@
   }
 
   ## if data type was supplied directly, always use that
-  if (!data_type %in% c("<i4", "<f8", "|S", "<U")) {
+  if (!data_type %in% c("|i1", "<i2", "<i4", "<f4", "<f8", "|S", "<U", "|b1")) {
     data_type <- switch(
       data_type,
       "integer" = "<i4",
       "double" = "<f8",
       "character" = "|S",
+      "logical" = "|b1",
       NULL
     )
   }
 
   if (is.null(data_type)) {
-    stop("Currently only able to write integer, double, and character arrays")
+    stop(
+      "Currently only able to write integer, double, character and logical arrays"
+    )
   }
 
   ## set a default fill value if needed
   if (missing(fill_value)) {
     fill_value <- switch(
       data_type,
+      "|i1" = 0L,
+      "<i2" = 0L,
       "<i4" = 0L,
+      "<f4" = 0,
       "<f8" = 0,
       "|S" = "",
       "<U" = "",
+      "|b1" = FALSE,
       NULL
     )
   }
@@ -48,7 +55,6 @@
   return(list(data_type = data_type, fill_value = fill_value))
 }
 
-
 #' Create an (empty) Zarr array
 #'
 #' @param zarr_array_path Character vector of length 1 giving the path to the
@@ -58,10 +64,11 @@
 #' @param chunk_dim Dimensions of the array chunks. Should be a numeric vector
 #'   with the same length as the `dim` argument.
 #' @param data_type Character vector giving the data type of the new array.
-#'   Currently this is limited to standard R data types.  Valid options are:
-#'   "integer", "double", "character".  You can also use the analogous NumpPy
-#'   formats: "<i4", "<f8", "|S".  If this argument isn't provided the
-#'   `fill_value` will be used to determine the datatype.
+#'   Valid options are: "integer", "double", "character", "logical", which are
+#'   based on standard R data types. You can also use the analogous Numpy
+#'   formats: "|i1", "<i2", "<i4", "<f4", "<f8", "|S", "|b1".
+#'   If this argument isn't provided the `fill_value` will be used to determine
+#'   the datatype.
 #' @param order Define the layout of the bytes within each chunk.  Valid options
 #'   are 'column', 'row', 'F' & 'C'.  'column' or 'F' will specify
 #'   "column-major" ordering, which is how R arrays are arranged in memory.
@@ -164,6 +171,7 @@ write_zarr_array <- function(
   x,
   zarr_array_path,
   chunk_dim,
+  data_type = storage.mode(x),
   order = "F",
   compressor = use_zlib(),
   fill_value,
@@ -180,7 +188,7 @@ write_zarr_array <- function(
     zarr_array_path = path,
     dim = dim(x),
     chunk_dim = chunk_dim,
-    data_type = storage.mode(x),
+    data_type = data_type,
     order = order,
     fill_value = fill_value,
     compressor = compressor,
@@ -193,6 +201,15 @@ write_zarr_array <- function(
   chunk_names <- .generate_chunk_names(x_dim = dim(x), chunk_dim = chunk_dim)
   chunk_ids <- apply(chunk_names, 1, paste0, collapse = dimension_separator)
 
+  same_type_lower_bytesize <- metadata$dtype %in% c("|i1", "<i2", "<f4")
+  lower_bytesize_type <- storage.mode(x) == "double" &&
+    metadata$dtype == "<i4"
+
+  can_overflow <- same_type_lower_bytesize || lower_bytesize_type
+  if (can_overflow) {
+    x <- .truncate_overflow(x, metadata$dtype)
+  }
+
   ## iterate over each chunk
   ## TODO: maybe this can be done in parallel with bplapply() ?
   res <- lapply(
@@ -246,10 +263,6 @@ write_zarr_array <- function(
     chunk_in_mem <- temp_chunk
   }
 
-  if (metadata$order == "C") {
-    chunk_in_mem <- aperm(chunk_in_mem)
-  }
-
   ## check the chunk path exists, and create if not
   if (isFALSE(dir.exists(dirname(chunk_path)))) {
     dir.create(dirname(chunk_path), recursive = TRUE, showWarnings = FALSE)
@@ -258,6 +271,7 @@ write_zarr_array <- function(
   .compress_and_write_chunk(
     input_chunk = chunk_in_mem,
     chunk_path = chunk_path,
+    metadata,
     compressor = metadata$compressor,
     data_type_size = .parse_datatype(metadata$dtype)$nbytes
   )
@@ -310,6 +324,7 @@ update_zarr_array <- function(zarr_array_path, x, index) {
     "integer" = "<i",
     "double" = "<f",
     "character" = c("|S", "<U", ">U"),
+    "logical" = "|b",
     NULL
   )
   if (!substr(metadata$dtype, 1, 2) %in% data_type) {
@@ -415,6 +430,7 @@ update_zarr_array <- function(zarr_array_path, x, index) {
   .compress_and_write_chunk(
     input_chunk = chunk_in_mem,
     chunk_path = chunk_path,
+    metadata,
     compressor = metadata$compressor,
     data_type_size = metadata$datatype$nbytes,
     is_base64 = (metadata$datatype$base_type == "unicode")
@@ -445,10 +461,18 @@ update_zarr_array <- function(zarr_array_path, x, index) {
 .compress_and_write_chunk <- function(
   input_chunk,
   chunk_path,
+  metadata,
   compressor = use_zlib(),
   data_type_size,
   is_base64 = FALSE
 ) {
+  if (metadata$order == "C") {
+    input_chunk <- codec_transpose_decode(
+      input_chunk,
+      indices = rev(seq_along(dim(input_chunk)))
+    )
+  }
+
   ## the compression tools need a raw vector
   raw_chunk <- .as_raw(
     as.vector(input_chunk),
@@ -456,6 +480,15 @@ update_zarr_array <- function(zarr_array_path, x, index) {
     is_base64 = is_base64
   )
 
+  # Endianness in unicode is handled during the conversion to/from
+  if (metadata$datatype$base_type != "unicode") {
+    raw_chunk <- codec_endian_encode(
+      raw_chunk,
+      endian = metadata$datatype$endian,
+      metadata$datatype$nbytes
+    )
+  }
+
   if (is.null(compressor)) {
     compressed_chunk <- raw_chunk
   } else if (compressor$id == "blosc") {
@@ -481,7 +514,10 @@ update_zarr_array <- function(zarr_array_path, x, index) {
     ## numpy stores the original size of the buffer in the first 4 bytes after
     ## compression. We should do that too for compatibility
     ## TODO: probably faster to do this in C and avoid copying the vector
-    compressed_chunk <- c(.as_raw(length(raw_chunk)), compressed_chunk)
+    compressed_chunk <- c(
+      .as_raw(length(raw_chunk), nchar = 4),
+      compressed_chunk
+    )
   } else if (compressor$id == "zstd") {
     compressed_chunk <- .Call(
       "compress_chunk_ZSTD",
@@ -523,7 +559,7 @@ update_zarr_array <- function(zarr_array_path, x, index) {
       )
     )
   } else {
-    writeBin(d, raw())
+    writeBin(d, raw(), size = nchar, endian = "little")
   }
 }
 
